package com.alihealth.d2d.provtest;

import cn.hutool.http.HttpRequest;
import com.alibaba.fastjson.JSONObject;
import com.alihealth.d2d.provtest.utils.Base64;
import com.alihealth.d2d.provtest.utils.EventPojo;
import com.alihealth.d2d.provtest.utils.KeyStoreUtil;
import com.alihealth.d2d.provtest.utils.StreamUtil;

import javax.crypto.BadPaddingException;
import javax.crypto.Cipher;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.security.*;
import java.security.interfaces.RSAPublicKey;
import java.security.spec.InvalidKeySpecException;
import java.security.spec.X509EncodedKeySpec;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * @ClassName ProvTestClient
 * @Desc 省局请求 客户端  请求端
 * @Author houduo.wk
 * @Date 2020/3/13
 **/
public class ProvTestClient {

    /**
     * 请求url
     */
    private static final String url = "http://localhost:8888/prov/test/receive";
    /**
     * 证书地址
     */
    private static final String PFX_FILE_PATH = "D:\\ynyp\\药品追溯\\d2d\\yunnan.pfx";
    /**
     * 证书密码
     */
    private static final String PFX_PASSWORD = "12345678";
    /**
     * 加密算法
     */
    private static final String algorithm = "RSA";


    public static void main(String[] args) throws Exception {
        //入参
        String fileName = UUID.randomUUID().toString().replaceAll("-", "").substring(0, 20);
        //需要同步的xml
        String data = StreamUtil.getStreamContent(fileName, getData());
        //流水号
        String processId = UUID.randomUUID().toString().replaceAll("-", "").substring(0, 20);
        //事件编号
        String eventId = UUID.randomUUID().toString().replaceAll("-", "").substring(0, 20);
        //数据文件内容主业务类型 10：基础信息数据
        //20：应用信息数据
        String eventType = "10";
        //数据文件内容子业务类型  详见：数据类型字典表
        String subType = "1011";

        //获取公钥
        String publicKey = getPublicKey();
        System.out.println("publicKey:"+ publicKey);
        String privateKey = getPrivateKey();
        System.out.println("privateKey:"+ privateKey);
        //公钥加密
        String en = testEncrypt(publicKey, data);
        System.out.println("publicKey加密后en:" + en);

        EventPojo eventPojo = new EventPojo();
        eventPojo.setData(en);
        eventPojo.setSubType(subType);

        //请求时间戳
        String timestamp = formatDateToyyyyMMddHHmm(new Date());

        //请求时间戳
        String ver = "1.0";

        Map<String, String> paramMap = new HashMap<>();
        paramMap.put("process_id", processId);
        paramMap.put("event_id", eventId);
        paramMap.put("event_type", eventType);
        paramMap.put("event", JSONObject.toJSONString(eventPojo));
        paramMap.put("timestamp", timestamp);
        paramMap.put("ver", ver);
        Map<String,Object> requestMap= new HashMap<>(4);
        requestMap.put("req", paramMap);
        System.out.println(JSONObject.toJSONString(requestMap));
        String body = HttpRequest.post(url).body(JSONObject.toJSONString(requestMap)).timeout(10000).execute().body();
        System.out.println(body);



    }

    /**
     * 公钥加密
     * @param key
     * @param data
     * @return
     * @throws NoSuchAlgorithmException
     * @throws InvalidKeySpecException
     * @throws NoSuchPaddingException
     * @throws IllegalBlockSizeException
     * @throws BadPaddingException
     * @throws InvalidKeyException
     * @throws IOException
     */
    public static String testEncrypt(String key,String data) throws NoSuchAlgorithmException, InvalidKeySpecException, NoSuchPaddingException, IllegalBlockSizeException, BadPaddingException, InvalidKeyException, IOException{
        byte[] decode = java.util.Base64.getDecoder().decode(key);
        RSAPublicKey pubKey = (RSAPublicKey) KeyFactory.getInstance(algorithm).generatePublic(new X509EncodedKeySpec(decode));
        //RSA加密
        Cipher ci = Cipher.getInstance(algorithm);
        ci.init(Cipher.ENCRYPT_MODE, pubKey);


        byte[] bytes = data.getBytes();
        int inputLen = bytes.length;
        int offLen = 0;//偏移量
        int i = 0;
        ByteArrayOutputStream bops = new ByteArrayOutputStream();
        while(inputLen - offLen > 0){
            byte [] cache;
            if(inputLen - offLen > 117){
                cache = ci.doFinal(bytes, offLen,117);
            }else{
                cache = ci.doFinal(bytes, offLen,inputLen - offLen);
            }
            bops.write(cache);
            i++;
            offLen = 117 * i;
        }
        bops.close();
        byte[] encryptedData = bops.toByteArray();
        String encodeToString = java.util.Base64.getEncoder().encodeToString(encryptedData);
        return encodeToString;
    }

    /**
     * 获取私钥
     * @return
     * @throws Exception
     */
    private static String getPrivateKey() throws Exception {
        KeyStore keyStore = KeyStoreUtil.loadKetStore(PFX_FILE_PATH,PFX_PASSWORD);
        return Base64.encode(KeyStoreUtil.getPrivateKey(keyStore, PFX_PASSWORD).getEncoded());
    }

    /**
     * 获取公钥
     * @return
     * @throws Exception
     */
    private static String getPublicKey() throws Exception {
        KeyStore keyStore = KeyStoreUtil.loadKetStore(PFX_FILE_PATH,PFX_PASSWORD);
        PublicKey publicKey = KeyStoreUtil.getPublicKey(keyStore);
        return Base64.encode(publicKey.getEncoded());
    }
    /**
     * 返回时间字符串, 可读形式的, yy年M月d日HH:mm 格式.
     *
     * @return - String 格式化后的时间
     */
    public static String formatDateToyyyyMMddHHmm(Date date) {
        if (date == null) {
            return "";
        }

        SimpleDateFormat dateFormat = new SimpleDateFormat(
                "yyyyMMddHHmmss");

        return dateFormat.format(date);
    }
    /**
     * 获取用于传输的xml
     * @return
     */
//    private static String getData(){
//        return "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n" +
//                "\n" +
//                "<VTTSBasic      xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n" +
//                "                xsi:noNamespaceSchemaLocation=\"1011.xsd\">\n" +
//                "\n" +
//                "        <datasetName>疫苗生产企业基本信息数据子集</datasetName>\n" +
//                "        <dataset>\n" +
//                "            <data>\n" +
//                "                <TYSHXYDM>912101131181555503</TYSHXYDM>\n" +
//                "                <QYMC>辽宁依生生物制药有限公司</QYMC>\n" +
//                "                <QYLX>有限公司</QYLX>\n" +
//                "                <ZSDZ>新城子区财落镇大辛二村</ZSDZ>\n" +
//                "                <ZSDZGJHDQ>156</ZSDZGJHDQ>\n" +
//                "                <ZSDZSZXSZZQ>210000</ZSDZSZXSZZQ>\n" +
//                "                <ZSDZSQZZZM>210100</ZSDZSQZZZM>\n" +
//                "                <ZSDZXZZXXJS>000044</ZSDZXZZXXJS>\n" +
//                "                <ZSDZXZJDBSC>新城子区财落镇大辛二村</ZSDZXZJDBSC>\n" +
//                "                <ZSDZCJLLD>新城子区财落镇大辛二村</ZSDZCJLLD>\n" +
//                "                <ZSDZMPHM>新城子区财落镇大辛二村</ZSDZMPHM>\n" +
//                "                <FDDBR>张译</FDDBR>\n" +
//                "                <ZCZB>4105.512万</ZCZB>\n" +
//                "                <CLRQ>19190101</CLRQ>\n" +
//                "                <YYQX>2005-06-29至无固定期限</YYQX>\n" +
//                "                <JYFW>无</JYFW>\n" +
//                "                <DJJG>沈阳市沈北新区市场监督管理局</DJJG>\n" +
//                "                <GDDHHM>无</GDDHHM>\n" +
//                "                <CZHM>无</CZHM>\n" +
//                "                <DZXX>无</DZXX>\n" +
//                "                <QYWZ>无</QYWZ>\n" +
//                "                <LXR>无</LXR>\n" +
//                "                <LXDH>无</LXDH>\n" +
//                "                <ZSGPS></ZSGPS>  </data>\n" +
//                "\n" +
//                "\n" +
//                "            <data>\n" +
//                "                <TYSHXYDM>91120116681888972M</TYSHXYDM>\n" +
//                "                <QYMC>康希诺生物股份公司</QYMC>\n" +
//                "                <QYLX>股份有限公司(中外合资、上市)</QYLX>\n" +
//                "                <ZSDZ>天津经济技术开发区西区南大街185号西区生物医药园四层401-420号</ZSDZ>\n" +
//                "                <ZSDZGJHDQ>156</ZSDZGJHDQ>\n" +
//                "                <ZSDZSZXSZZQ>120000</ZSDZSZXSZZQ>\n" +
//                "                <ZSDZSQZZZM>120000</ZSDZSQZZZM>\n" +
//                "                <ZSDZXZZXXJS>000035</ZSDZXZZXXJS>\n" +
//                "                <ZSDZXZJDBSC>西区南大街185号西区生物医药园四层401-420号</ZSDZXZJDBSC>\n" +
//                "                <ZSDZCJLLD>西区南大街185号西区生物医药园四层401-420号</ZSDZCJLLD>\n" +
//                "                <ZSDZMPHM>生物医药园四层401-420号</ZSDZMPHM>\n" +
//                "                <FDDBR>YU XUEFENG（宇学峰）</FDDBR>\n" +
//                "                <ZCZB>77706.99万</ZCZB>\n" +
//                "                <CLRQ>19910101</CLRQ>\n" +
//                "                <YYQX>2009-01-13至2059-01-12</YYQX>\n" +
//                "                <JYFW>无</JYFW>\n" +
//                "                <DJJG>天津市滨海新区市场和质量监督管理局</DJJG>\n" +
//                "                <GDDHHM>无</GDDHHM>\n" +
//                "                <CZHM>无</CZHM>\n" +
//                "                <DZXX>无</DZXX>\n" +
//                "                <QYWZ>www.cansinotech.com</QYWZ>\n" +
//                "                <LXR>无</LXR>\n" +
//                "                <LXDH>无</LXDH>\n" +
//                "                <ZSGPS></ZSGPS>  </data>\n" +
//                "\n" +
//                "\n" +
//                "\n" +
//                "            <data>\n" +
//                "                <TYSHXYDM>9132129168412218X6</TYSHXYDM>\n" +
//                "                <QYMC>江苏金迪克生物技术有限公司</QYMC>\n" +
//                "                <QYLX>有限公司</QYLX>\n" +
//                "                <ZSDZ>泰州市郁金路12号</ZSDZ>\n" +
//                "                <ZSDZGJHDQ>156</ZSDZGJHDQ>\n" +
//                "                <ZSDZSZXSZZQ>320000</ZSDZSZXSZZQ>\n" +
//                "                <ZSDZSQZZZM>321200</ZSDZSQZZZM>\n" +
//                "                <ZSDZXZZXXJS>000025</ZSDZXZZXXJS>\n" +
//                "                <ZSDZXZJDBSC>郁金路12号</ZSDZXZJDBSC>\n" +
//                "                <ZSDZCJLLD>郁金路12号</ZSDZCJLLD>\n" +
//                "                <ZSDZMPHM>郁金路12号</ZSDZMPHM>\n" +
//                "                <FDDBR>余军</FDDBR>\n" +
//                "                <ZCZB>无</ZCZB>\n" +
//                "                <CLRQ>19900101</CLRQ>\n" +
//                "                <YYQX>无</YYQX>\n" +
//                "                <JYFW>无</JYFW>\n" +
//                "                <DJJG>无</DJJG>\n" +
//                "                <GDDHHM>无</GDDHHM>\n" +
//                "                <CZHM>无</CZHM>\n" +
//                "                <DZXX>无</DZXX>\n" +
//                "                <QYWZ>无</QYWZ>\n" +
//                "                <LXR>无</LXR>\n" +
//                "                <LXDH>无</LXDH>\n" +
//                "                <ZSGPS></ZSGPS>  </data>\n" +
//                "\n" +
//                "\n" +
//                "            <data>\n" +
//                "                <TYSHXYDM>912200007307650053</TYSHXYDM>\n" +
//                "                <QYMC>吉林亚泰生物药业股份有限公司</QYMC>\n" +
//                "                <QYLX>股份有限公司</QYLX>\n" +
//                "                <ZSDZ>吉林省长春市高新开发区北湖科技开发区高科技中心B区537-A室</ZSDZ>\n" +
//                "                <ZSDZGJHDQ>156</ZSDZGJHDQ>\n" +
//                "                <ZSDZSZXSZZQ>220000</ZSDZSZXSZZQ>\n" +
//                "                <ZSDZSQZZZM>220100</ZSDZSQZZZM>\n" +
//                "                <ZSDZXZZXXJS>000000</ZSDZXZZXXJS>\n" +
//                "                <ZSDZXZJDBSC>高新开发区北湖科技开发区高科技中心B区537-A室</ZSDZXZJDBSC>\n" +
//                "                <ZSDZCJLLD>高新开发区北湖科技开发区高科技中心B区537-A室</ZSDZCJLLD>\n" +
//                "                <ZSDZMPHM>高科技中心B区537-A室</ZSDZMPHM>\n" +
//                "                <FDDBR>刘晓峰</FDDBR>\n" +
//                "                <ZCZB>43564.770000万</ZCZB>\n" +
//                "                <CLRQ>19940307</CLRQ>\n" +
//                "                <YYQX>2010-06-30至2020-12-31</YYQX>\n" +
//                "                <JYFW>小容量注射剂（卡介菌多糖核酸注射液）、疫苗[人用狂犬病疫苗（地鼠肾细胞）]生产（药品生产许可证有效期至2020年12月31日）及技术咨询、技术服务、研究开发；医疗器械研发、生产和销售；第Ⅰ类、第Ⅱ类、第Ⅲ类体外诊断试剂研发、生产及销售；消毒产品（危险化学品除外）的研发、生产及销售。（依法须经批准的项目，经相关部门批准后方可开展经营活动）</JYFW>\n" +
//                "                <DJJG>吉林省市场监督管理厅</DJJG>\n" +
//                "                <GDDHHM>无</GDDHHM>\n" +
//                "                <CZHM>无</CZHM>\n" +
//                "                <DZXX>无</DZXX>\n" +
//                "                <QYWZ>无</QYWZ>\n" +
//                "                <LXR>无</LXR>\n" +
//                "                <LXDH>无</LXDH>\n" +
//                "                <ZSGPS></ZSGPS>\n" +
//                "            </data>\n" +
//                "        </dataset>\n" +
//                "    </VTTSBasic>\n";
//    }


    private static String getData(){
        return "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n" +
                "\n" +
                "     <VTTSBasic>\n" +
                "<datasetName>境内药品生产企业基本信息数据子集</datasetName>\n" +
                "<dataset>\n" +
                "<data>\n" +
                "<TYSHXYDM>911100007635353107</TYSHXYDM>\n" +
                "<JNYPSCQYMC>上海药业有限公司</JNYPSCQYMC>\n" +
                "<JNYPSCQYLX>股份有限公司</JNYPSCQYLX>\n" +
                "<ZSDZ>吉林省长春市高新开发区北湖科技开发区高科技中心B区537-A室 </ZSDZ>\n" +
                "<ZSDZGJHDQ>156</ZSDZGJHDQ>\n" +
                "<ZSDZSZXSZZQ>220000</ZSDZSZXSZZQ>\n" +
                "<ZSDZSQZZZM>220100</ZSDZSQZZZM>\n" +
                "<ZSDZXZZXXJS>000000</ZSDZXZZXXJS>\n" +
                "<ZSDZXZJDBSC>高新开发区北湖科技开发区高科技中心B区537-A室 </ZSDZXZJDBSC>\n" +
                "<ZSDZCJLLD>高新开发区北湖科技开发区高科技中心B区537-A室</ZSDZCJLLD>\n" +
                "<ZSDZMPHM>高科技中心B区537-A室</ZSDZMPHM>\n" +
                "<FDDBR>刘晓峰</FDDBR>\n" +
                "<ZCZB>43564.770000万</ZCZB>\n" +
                "<CLRQ>19940307</CLRQ>\n" +
                "<YYQX>2010-06-30至2020-12-31</YYQX>\n" +
                "<JYFW>小容量注射剂</JYFW>\n" +
                "<DJJG>上海市闵行区向阳路888号</DJJG>\n" +
                "<GDDHHM>0531-8291****</GDDHHM>\n" +
                "<CZHM>0531-8291****</CZHM>\n" +
                "<DZXX>***@dsyywl.com</DZXX>\n" +
                "<QYWZ>www.dsyywl.com.cn</QYWZ>\n" +
                "<LXR>张三</LXR>\n" +
                "<LXDH>0531-8291****</LXDH>\n" +
                "<ZSGPS>113.54,22.19</ZSGPS>\n" +
                "</data>\n" +
                "<data>\n" +
                "<TYSHXYDM>911100007635353107</TYSHXYDM>\n" +
                "<JNYPSCQYMC>上海药业有限公司</JNYPSCQYMC>\n" +
                "<JNYPSCQYLX>股份有限公司</JNYPSCQYLX>\n" +
                "<ZSDZ>吉林省长春市高新开发区北湖科技开发区高科技中心B区537-A室 </ZSDZ>\n" +
                "<ZSDZGJHDQ>156</ZSDZGJHDQ>\n" +
                "<ZSDZSZXSZZQ>220000</ZSDZSZXSZZQ>\n" +
                "<ZSDZSQZZZM>220100</ZSDZSQZZZM>\n" +
                "<ZSDZXZZXXJS>000000</ZSDZXZZXXJS>\n" +
                "<ZSDZXZJDBSC>高新开发区北湖科技开发区高科技中心B区537-A室 </ZSDZXZJDBSC>\n" +
                "<ZSDZCJLLD>高新开发区北湖科技开发区高科技中心B区537-A室</ZSDZCJLLD>\n" +
                "<ZSDZMPHM>高科技中心B区537-A室</ZSDZMPHM>\n" +
                "<FDDBR>刘晓峰</FDDBR>\n" +
                "<ZCZB>43564.770000万</ZCZB>\n" +
                "<CLRQ>19940307</CLRQ>\n" +
                "<YYQX>2010-06-30至2020-12-31</YYQX>\n" +
                "<JYFW>小容量注射剂</JYFW>\n" +
                "<DJJG>上海市闵行区向阳路888号</DJJG>\n" +
                "<GDDHHM>0531-8291****</GDDHHM>\n" +
                "<CZHM>0531-8291****</CZHM>\n" +
                "<DZXX>***@dsyywl.com</DZXX>\n" +
                "<QYWZ>www.dsyywl.com.cn</QYWZ>\n" +
                "<LXR>张三</LXR>\n" +
                "<LXDH>0531-8291****</LXDH>\n" +
                "<ZSGPS>113.54,22.19</ZSGPS>\n" +
                "</data>\n" +
                "</dataset>\n" +
                "</VTTSBasic>  \n";
    }



}
